@@ -125,7 +125,7 @@ print.src_spark <- function(x, ...) {
 db_save_query.spark_connection <- function(con, sql, name, temporary = TRUE, ...)
 {
   df <- spark_dataframe(con, sql)
-  invoke(df, "registerTempTable", name)
+  sdf_register(df, name)
 
   # compute() is expected to preserve the query, cache as the closest mapping.
   if (!sparklyr_boolean_option("sparklyr.dplyr.compute.nocache"))
