@@ -12,6 +12,9 @@ sample_n.tbl_spark <- function(tbl,
                                replace = FALSE,
                                weight = NULL,
                                .env = parent.frame()) {
+  if (spark_version(spark_connection(tbl)) < "2.0.0")
+      stop("sample_n() is not supported until Spark 2.0 or later")
+
   add_op_single("sample_n", .data = tbl, args = list(
     size = size,
     replace = replace,
@@ -28,6 +31,9 @@ sample_frac.tbl_spark <- function(tbl,
                                   replace = FALSE,
                                   weight = NULL,
                                   .env = parent.frame()) {
+  if (spark_version(spark_connection(tbl)) < "2.0.0")
+    stop("sample_frac() is not supported until Spark 2.0 or later")
+
   add_op_single("sample_frac", .data = tbl, args = list(
     size = size,
     replace = replace,
