@@ -32,7 +32,12 @@ spark_schema_from_rdd <- function(sc, rdd, column_names) {
     stop("Failed to infer column types, please use explicit types.")
 
   fields <- lapply(seq_along(colTypes), function(idx) {
-    name <- if (is.null(column_names)) as.character(idx) else column_names[[idx]]
+    name <- if (is.null(column_names))
+      as.character(idx)
+    else if (idx <= length(column_names))
+      column_names[[idx]]
+    else
+      paste0("X", idx)
 
     invoke_static(
       sc,
@@ -63,7 +68,8 @@ spark_schema_from_rdd <- function(sc, rdd, column_names) {
 #'   \code{groupN} contain the values of the \code{group_by} values. When
 #'   \code{group_by} is not specified, \code{f} takes only one argument.
 #' @param columns A vector of column names or a named vector of column types for
-#'   the transformed object. Defaults to the names from the original object.
+#'   the transformed object. Defaults to the names from the original object and
+#'   adds indexed column names when not enough columns are specified.
 #' @param memory Boolean; should the table be cached into memory?
 #' @param group_by Column name used to group by data frame partitions.
 #' @param packages Boolean; distribute \code{.libPaths()} packages to nodes?
@@ -78,13 +84,16 @@ spark_apply <- function(x,
                         packages = TRUE,
                         ...) {
   args <- list(...)
+  assert_that(is.function(f))
+
   sc <- spark_connection(x)
   sdf <- spark_dataframe(x)
   sdf_columns <- colnames(x)
   rdd_base <- invoke(sdf, "rdd")
   grouped <- !is.null(group_by)
   args <- list(...)
-  rlang <- spark_config_value(x$sc, "sparklyr.closures.rlang", FALSE)
+  rlang <- spark_config_value(sc, "sparklyr.closures.rlang", FALSE)
+  proc_env <- connection_config(sc, "sparklyr.apply.env.")
 
   # backward compatible support for names argument from 0.6
   if (!is.null(args$names)) {
@@ -96,7 +105,7 @@ spark_apply <- function(x,
   if (rlang) warning("The `rlang` parameter is under active development.")
 
   # disable package distribution for local connections
-  if (spark_master_is_local(sc$master)) packages = FALSE
+  if (spark_master_is_local(sc$master)) packages <- FALSE
 
   # create closure for the given function
   closure <- serialize(f, NULL)
@@ -165,7 +174,9 @@ spark_apply <- function(x,
     as.list(sdf_columns),
     as.list(group_by),
     closure_rlang,
-    bundle_path
+    bundle_path,
+    as.environment(proc_env),
+    as.integer(60)
   )
 
   # while workers need to relaunch sparklyr backends, cache by default
@@ -185,3 +196,16 @@ spark_apply_rlang_serialize <- function() {
   else
     rlang_serialize
 }
+
+#' Log Writter for Spark Apply
+#'
+#' Writes data to log under \code{spark_apply()}.
+#'
+#' @param ... Arguments to write to log.
+#' @param level Severity level for this entry; recommended values: \code{INFO},
+#'   \code{ERROR} or \code{WARN}.
+#'
+#' @export
+spark_apply_log <- function(..., level = "INFO") {
+  worker_log_level(..., level = level, component = "Closure")
+}
