@@ -12,7 +12,9 @@ class WorkerRDD(
   port: Int,
   groupBy: Array[String],
   closureRLang: Array[Byte],
-  bundlePath: String
+  bundlePath: String,
+  customEnv: Map[String, String],
+  connectionTimeout: Int
   ) extends RDD[Row](prev) {
 
   private[this] var exception: Option[Exception] = None
@@ -58,7 +60,8 @@ class WorkerRDD(
 
     backend.init(
       port,
-      sessionId
+      sessionId,
+      connectionTimeout
     )
 
     backendPort = backend.getPort()
@@ -86,7 +89,13 @@ class WorkerRDD(
           logger.log("is starting rscript")
 
           val rscript = new Rscript(logger)
-          rscript.init(sessionId, backendPort, config)
+          rscript.init(
+            sessionId,
+            backendPort,
+            config,
+            customEnv
+          )
+
           lock.synchronized {
             lock.notify
           }
